var FdFile = {
  getImagesDirectory: function() {
    var file = Components.classes["@mozilla.org/file/directory_service;1"]
               .getService(Components.interfaces.nsIProperties)
               .get("ProfD", Components.interfaces.nsIFile);
    file.append("fastdial");
    if (!file.exists()) file.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0777);
    return file;
  },

  getImageName: function(id) {
    return id + ".png";
  }, 

  getImageFile: function(id) {
    var file = FdFile.getImagesDirectory();
    file.append(FdFile.getImageName(id));
    return file;
  },

  deleteImage: function(id) {
    var file = FdFile.getImagesDirectory();
    file.append(FdFile.getImageName(id));
    try {
      file.remove(false);
    } catch(e) {}
  },

  saveImage: function(id, imageURL) {
    var file = FdFile.getImageFile(id);
    var out = FdFile.createOutputStream(file);
    var data = atob(imageURL.replace(/data:image\/png;base64,/, ""));
    out.write(data, data.length);
    out.close();
  },

  getImage: function(id) {
    var file = FdFile.getImageFile(id);
    try {
      return "data:image/png;base64," + btoa(FdFile.readBinaryFile(file));
    } catch(e) {
      return "";
    }
  },

  createInputStream: function(file) {
    var stream = Components.classes["@mozilla.org/network/file-input-stream;1"]
                 .createInstance(Components.interfaces.nsIFileInputStream);
    stream.init(file, -1, 0, 0);
    var scriptable = Components.classes["@mozilla.org/scriptableinputstream;1"]
                     .createInstance(Components.interfaces.nsIScriptableInputStream);
    scriptable.init(stream); 
    return { stream: stream,
             scriptable: scriptable,
             readFully: function() {
               return this.scriptable.read(this.scriptable.available());
             },
             close: function() {
               this.scriptable.close();
               this.stream.close();
             }
           };
  },

  createOutputStream: function(file) {
    var stream = Components.classes["@mozilla.org/network/file-output-stream;1"]
                 .createInstance(Components.interfaces.nsIFileOutputStream);
    stream.init(file, 0x04 | 0x08 | 0x20, 0666, 0); // read & write, create, truncate
    return stream;
  },

  readFile: function(file) {
    if (file instanceof Components.interfaces.nsIFile) nsiFile = file;
    else {
      var nsiFile = Components.classes["@mozilla.org/file/local;1"]
                    .getService(Components.interfaces.nsILocalFile);
      nsiFile.initWithPath(file);
    }
    var inn = FdFile.createInputStream(nsiFile);
    try {
      return inn.readFully();
    }
    finally {
      inn.close();
    }
  },

  readBinaryFile: function(file) {
    var istream = Components.classes["@mozilla.org/network/file-input-stream;1"]
                  .createInstance(Components.interfaces.nsIFileInputStream);
    istream.init(file, -1, -1, false);

    var bstream = Components.classes["@mozilla.org/binaryinputstream;1"]
                  .createInstance(Components.interfaces.nsIBinaryInputStream);
    bstream.setInputStream(istream);
    return bstream.readBytes(bstream.available());
  },

  getExtensionDirectory: function() {
    var id = "fastdial@telega.phpnet.us";
    return Components.classes["@mozilla.org/extensions/manager;1"]
           .getService(Components.interfaces.nsIExtensionManager)
           .getInstallLocation(id)
           .getItemLocation(id);
  },

  getPredefinedImage: function(name) {
    var file = FdFile.getExtensionDirectory();
    file.append("chrome");
    file.append("skin");
    file.append("classic");
    file.append(name);
    return "data:image/png;base64," + btoa(FdFile.readBinaryFile(file));
  },

  exchangeImages: function(sourceId, targetId) {
    var sourceFile = FdFile.getImageFile(sourceId);
    var sourceExists = sourceFile.exists();
    var targetFile = FdFile.getImageFile(targetId);
    var targetExists = targetFile.exists();
    if (sourceExists && targetExists) {
      sourceFile.moveTo(null, "tmp" + new Date().getTime());
    }
    if (targetExists) targetFile.moveTo(null, FdFile.getImageName(sourceId));
    if (sourceExists) sourceFile.moveTo(null, FdFile.getImageName(targetId));
  }
}
